/*--------------------------------------------------------------------

    This is where all the magic is going to take place. This
    is our main file, and everything is going to happen through here

    Tutorial #8
        changes:
            - Everything is a change. This is a new class. Study it
              and learn it!! BY HEART!!!
---------------------------------------------------------------------*/


#include "DXFont.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDXFont::CDXFont()
{
    // Initialize the variables

    m_pFont = NULL;
    m_pD3DDevice = NULL;

    // Set the default color to white
    m_dwColor = 0xFFFFFFFF;

}

CDXFont::~CDXFont()
{
    // Simply destroy it
    DestroyFont();
}


BOOL CDXFont::InitializeFont(IDirect3DDevice8* pDevice, char* strFontName, int iFontSize)
{
    // First make sure we have a valid device
    if((m_pD3DDevice = pDevice) == NULL)
        return FALSE;

    // Then make sure the font name is good. If it isnt
    // specify one yourself
    if(strFontName == NULL)
        strFontName = "Ariel";

    // Then make sure teh font size is good
    if(iFontSize == 0)
        iFontSize = 15;

    // Now we need to fill out a structure called LOGFONT
    LOGFONT lf;
    // Clear teh structure
    ZeroMemory(&lf, sizeof(LOGFONT));

    // Now fill it in with stuff.

    strcpy(lf.lfFaceName, strFontName);     // Copy teh font name into teh appropriate
                                            // place. lfFontFace is what type of font you
                                            // want to create ie: ariel, tahoma, etc...

    lf.lfHeight = -(iFontSize);             // For this you have to give it a negative
                                            // number of the font size that you want
                                            // If you want font size 10, then pass in -10

    // Ok that's all we're going to be filling. So now we create teh font object. There
    // is a lot more to this LOGFONT structure then what I've went over. You should
    // experiment with it. Set teh italics member, the boldness, and a few other things
    // and see what happens.

    if(FAILED(D3DXCreateFontIndirect(m_pD3DDevice,      // Just give it the device for it's
                                                        // own internal use

                                     &lf,               // Pass in the LOGFONT structure
                                                        // that we (kinda) filled out 

                                     &m_pFont)))        // The address of our font interface
                                                        // that is going to be initialized
                                                        // for us
        return FALSE;

    // If we've reached here, then we have a valid font interface and we can start drawing

    return TRUE;
}

void CDXFont::DestroyFont()
{
    if(m_pFont != NULL)
    {
        m_pFont->Release();
        m_pFont = NULL;
    }

    // DO NOT release the device, because the rest
    // of the application might still be using it.
    // The device DOES NOT belong to this class. It
    // belongs to the base class (CMyApplication)
    // and we just have a pointer to the device.
    // so be careful with what you do with the device
    // If you release teh device from over here
    // then your application will probably crash on exit
    // Incfact, Im going to get rid of the device
    // pointer in the next app.
}

BOOL CDXFont::Begin()
{
    // You *should* call this before drawingt text, but you 
    // dont have to becayse the  DrawText() function calls it,
    // and End(), internally. But that would cost a performance
    // hit if your calling *our* Print() function more then
    // once per frame.
    if(FAILED(m_pFont->Begin()))
        return FALSE;

    return TRUE;
}

void CDXFont::End()
{
    // End the scene when you're finished drawing
    m_pFont->End();
}

// You must call this method between m_pD3DDevice->BeginScene() and
// m_pD3DDevice->EndScene() otherwise it will fail
BOOL CDXFont::Print(char* strText, int iXPos, int iYPos)
{
    // Make sure the fnt interface is good
    if(!m_pFont)
        return FALSE;

    // Now we have to declare a RECT structure and fill it, because that is what
    // the DrawText() fuinction takes to know which area to draw teh text in. So
    // we'll fill it in depending on teh x,y position that was specified during
    // this function call

    RECT box;
    box.bottom = iYPos + 2000;      // Make the rectangle wide enough to take up
                                    // teh whole screen

    box.left = iXPos;               // Text's X-Position

    box.right = iXPos + 2000;       // High enough for teh whole screen

    box.top = iYPos;                // Text's Y-Position

    // So now all we do is call DrawText to get the text on screen 

    if(FAILED(m_pFont->DrawText(strText,         // This is the text you want to print

                                -1,              // This paarameter tells DirectX the length
                                                 // of our string. If we use -1, then DX will
                                                 // assume that it si a null terminated string

                                &box,            // Address of our previously filled our
                                                 // RECT structure that tells DirectX which
                                                 // Area of eh screen to draw in.

                                DT_LEFT,         // This param tells DirectX how the text is
                                                 // to be formatted. I have specified LEFT, so
                                                 // that te text stays on teh left. You can use
                                                 // CENTER or anything else. Check MSDN for a 
                                                 // full list of valid formats. Also you can
                                                 // add a parameter to this function to specify 
                                                 // teh format at run time. I have only chosen
                                                 // to specify, x, y coordinates and teh actual
                                                 // string of text that you want printed

                                m_dwColor)))     // Color of the text!!
        return FALSE;

    return TRUE;
}

void CDXFont::SetFontColor(DWORD dwColor)
{
    // Simply set the color.
    m_dwColor = dwColor;
}

BOOL CDXFont::InvalidateFontObject()
{
    // This is easy just call this function and the 
    // font is invalidated. DX takes care of it
    if(FAILED(m_pFont->OnLostDevice()))
        return FALSE;

    return TRUE;
}

BOOL CDXFont::RestoreFontObject()
{
    // Again DX handles everything for you by simply
    // calling this one function
    if(FAILED(m_pFont->OnResetDevice()))
        return FALSE;

    return TRUE;
}

/*

    **Drawing Fonts**

      This calls is very simple and straight forward. Granted it's not teh best way to
      write text to teh screen using DirectX, there are better ways. If you made a bitmap
      with all teh letters in it, then you can have a font class that draws a portion of
      that bitmap on request. Your bitmap would look something like this

      -------------------
      |A|B|C|D|E|F|G|H|I|
      -------------------
      |J|K|L|M|N|O|P|Q|R|
      -------------------
      |S|T| etc...

      So if you wanted to write "ALI" you would access the 1st, 12th, and 9th tiles of the
      bitmap, and have them texture mapped to three squares made from a vertex buffer, then
      you just DrawPrimitive the square, with using SetTexture(FontBitmapTexture).

      That way would be better then using the D3DX interface, but also more of a hassle. 
      Using teh d3dx font interface is VERY simple, so that's what we'll use, maybe some other
      time Ill make a proper font engine.
*/











/**************************************************************************************
-Written/Submitted By Ali Fareed
-Comments/Questions/Suggestions: k9eks@hotmail.com
-Website: http://www28.brinkster.com/triplebuffer/
-This work is the authors work, and can be freely used

Triple Buffer and everyone that has anything to do
with Triple Buffer takes no responsibility for
anything that happens by using this code. ie: If this
code caues your computer to blow up or anything
then you are held responsible. hehe :).
***************************************************************************************/